' =========================================================================
'
'   File...... stepper_demo.spin
'   Purpose... Demonstrates stepper.spin object
'   Author.... Jon Williams, Copyright (C) 2006
'   E-mail.... jwilliams@efx-tek.com
'   Started... 17 JUN 2006
'   Updated... 05 NOV 2006
'              MJC - Added TV_Text Object for Display output
'
' =========================================================================
'
' Permission granted by author for anyone to use/modify/redistribute this
' code.  No warranty of suitability for your applicationis expressed or
' implied.
'
' Connections (for L293D DIP):
' --------------------------------------
' P0 -> L293D.1 and L293D.9
' P1 -> L293D.2, L293D.3   -> Phase 1 
' P2 -> L293D.7, L293D.6   -> Phase 2
' P3 -> L293D.10, L293D.11 -> Phase 3
' P4 -> L293D.15, L293D.14 -> Phase 4


CON

  _CLKMODE      = XTAL1 + PLL16X
  _XINFREQ      = 5_000_000

  EN            = 0                                     ' L293 enable inputs (both)
  M1            = EN + 1                                ' motor connections
  M4            = EN + 4                                ' -- (contiguous)

  STEPS_PER_REV = 200                                   ' steps per revolution
  MIN_STEP_MS   = 5                                     ' minimum step time
  
  ' Constants for Video Display Object
  VideoPins = 12                                        ' PINs where Video interface is installed
  VideoCls = 0                                          ' Video driver clear screen code
  CR = 13                                               ' ASCII <CR>
  LF = 10                                               ' ASCII <LF>
  

VAR

  long  idx


OBJ

  motor :       "stepper"                               ' Declare Stepper Motor object
  VideoDisplay: "tv_text"                               ' Declare Video Display object 


PUB Start                                                          
  
  VideoDisplay.Start(VideoPins)                         ' Instantiates Video Display
  VideoDisplay.Out(VideoCls)                            ' Clear Video Display  
  VideoDisplay.Str(String("Stepper Motor Demo", CR,CR)) ' Display Program Title on Video Display
  main
  
PUB main

  if motor.start(EN, MIN_STEP_MS)
    motor.enable(true)                                  ' enable driver
    motor.setRun(true)                                  ' start the motor
    motor.stepRun                                       ' set to step mode

    repeat idx from 1 to 5  
      VideoDisplay.Str(String("one revolution forward", CR)) 
      motor.setSteps(STEPS_PER_REV)                     ' one revolution forward
      repeat until (motor.getSteps == 0)
      pause(200)
      VideoDisplay.Str(String("one revolution reverse", CR))
      motor.setSteps(-STEPS_PER_REV)                    ' one revolution reverse
      repeat until (motor.getSteps == 0)  
      pause(200) 

    motor.setStepTmr(500 / STEPS_PER_REV)               ' set to 1/2 sec per revolution
    motor.freeRun                                       ' free run mode when enabled

    motor.setRun(false)                                 ' stop motor 
    motor.setDir(motor#RUN_FWD)                         ' set direction to forward
    motor.setRun(true)                                  ' start motor
    VideoDisplay.Str(String("forward for 5 seconds", CR))      
    pause(5_000)                                        ' run for 5 seconds
    
    motor.setRun(false)
    motor.setDir(motor#RUN_REV)
    motor.setRun(true)
    VideoDisplay.Str(String("reverse for 5 seconds", CR))
    pause(5_000)
    
    motor.setRun(false)
    VideoDisplay.Str(String("Motor Stopped, Reset to repeat"))  
    motor.stop                                          ' unload object
    

PRI pause(ms) | c

  c := cnt                                              ' sync with system counter
  repeat until (ms-- == 0)                              ' repeat while time left
    waitcnt(c += clkfreq / 1000)                        ' wait 1 ms
