' =========================================================================
'
'   File...... stepper.spin
'   Purpose... Stepper motor object
'   Author.... Jon Williams, Copyright (C) 2006
'   E-mail.... jwilliams@efx-tek.com
'   Started...
'   Updated... 18 JUN 2006
'
' =========================================================================
'
' Permission granted by author for anyone to use/modify/redistribute this
' code.  No warranty of suitability for your applicationis expressed or
' implied.


CON

  M_ENABLED = %0001                                     ' driver enabled / disabled 
  M_RUN     = %0010                                     ' run / hold
  M_STEPS   = %0100                                     ' do steps / free-run
  M_REV     = %1000                                     ' reverse / forward

  RUN_FWD   = 0
  RUN_REV   = 1


VAR

  long  cogon, cog                                      ' cog status
  long  stack[64]                                       ' local stack
        
  long  en, mode
  long  minStpTm, timer, stepTm, numSteps, stpIdx 


PUB start(ePin, mnTime) : okay

'' Load background stepper controller if cog available

  okay := cogon := (cog := cognew(runStepper(ePin + 1), @stack)) > 0

  if okay
    en := ePin                                          ' save enable pin
    minStpTm := stepTm := mnTime                        ' save minimum step time
    setMode(%0000)                                      ' set defaults
    enable(false)
    stpIdx~                                             ' clear step index
    numSteps~                                           ' clear steps


PUB stop

  if cogon~                                             ' if object running, mark stopped
    dira[en]~                                           '   float enable pin
    cogstop(cog)                                        '   stop the cog
 

PUB setMode(m)

  mode := m & %1111                                     ' update mode

  if (mode & M_ENABLED)                                 ' check L292D enable output
    enable(true)                                        '   enable
  else
    enable(false)                                       '   disable


PUB getMode

  return mode                                           ' return current mode


PUB enable(status)

  if status
    mode := mode | M_ENABLED                            ' set enable bit
    outa[en]~~                                          ' enable pin high
    dira[en]~~
  else
    mode := mode & !M_ENABLED                           ' clear enable bit
    outa[en]~                                           ' enable pin low
    dira[en]~~
  

PUB setRun(status)

  if status
    mode := mode | M_RUN                                ' set running bit
  else
    mode := mode & !M_RUN                               ' clear running bit


PUB setSteps(s)

  if (s => 0)                                           ' positive step count?
    numSteps := s                                       ' yes
    setDir(RUN_FWD)                                     '   run forward
  else
    numSteps := -s                                      ' no, make positive
    setDir(RUN_REV)                                     '   run in reverse


PUB getSteps

  return numSteps                                       ' return curren step count    


PUB stepRun

  mode := mode | M_STEPS                                ' set steps bit
  

PUB freeRun

  mode := mode & !M_STEPS                               ' clear steps bit
   

PUB setDir(d)

  if (d == RUN_FWD)
    mode := mode & !M_REV                               ' set to forward
  else
    mode := mode | M_REV                                ' set to reverse
  

PUB setStepTmr(time)

  stepTm := time #> minStpTm                            ' set new step time (with min)
  timer := stepTm                                       ' reset background timing


PUB getStepTmr

  return stepTm 

  
PRI runStepper(m1) | m4, c
 
'' Run stepper motor as "background" process
'' -- stepper.start method launches this method into separate cog

  m4 := m1 + 3                                          ' calculate last pin
  dira[m4..m1]~~                                        ' make outputs for this cog
 
  timer~                                                ' clear timer
  c := cnt                                              ' sync with system counter
  repeat                                                ' run until cog unloaded
    waitcnt(c += clkfreq / 1000)                        ' wait 1 ms
    if (mode & M_ENABLED) and (mode & M_RUN)            ' background action active?
      timer := ++timer // stepTm
      if (timer == 0)                                   ' ready for step?
        if (mode & M_STEPS)
          if (numSteps-- > 0)                           ' any steps left?
            if (mode & M_REV)                           ' check direction
              stpIdx := (stpIdx + 3) // 4               ' point to previous step
              outa[m4..m1] := Steps[stpIdx]
            else
              stpIdx := ++stpIdx // 4                   ' point to next step
              outa[m4..m1] := Steps[stpIdx]            
        else
          if (mode & M_REV)                             ' check direction
            stpIdx := (stpIdx + 3) // 4  
            outa[m4..m1] := Steps[stpIdx]
          else
            stpIdx := ++stpIdx // 4      
            outa[m4..m1] := Steps[stpIdx]         


DAT

  Steps       byte      %0011, %0110, %1100, %1001      ' step table

  